package javakirja;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;

/**
 * @author Mika Vesterholm
 * @version 1.0
 */

public class ButtonBean extends Canvas
 implements MouseListener, Serializable {

  private java.awt.Color textColor = Color.black;
  private String text = "";;
  private transient Vector buttonListeners;

  public ButtonBean() {
    addMouseListener(this);
  }

  public Color getTextColor() {
    return textColor;
  }
  public void setTextColor(Color newTextColor) {
    textColor = newTextColor;
    repaint();
  }

  public void setText(String newText) {
    text = newText;
    repaint();
  }
  public String getText() {
    return text;
  }

  public synchronized void removeButtonListener(ButtonListener l) {
    if (buttonListeners != null && buttonListeners.contains(l)) {
      Vector v = (Vector) buttonListeners.clone();
      v.removeElement(l);
      buttonListeners = v;
    }
  }
  public synchronized void addButtonListener(ButtonListener l) {
    Vector v = buttonListeners == null ? new Vector(2) : (Vector) buttonListeners.clone();
    if (!v.contains(l)) {
      v.addElement(l);
      buttonListeners = v;
    }
  }
  protected void fireButtonPressed(ButtonEvent e) {
    Vector listeners;
    synchronized(this) {
      listeners = buttonListeners;
    }
    if (listeners != null) {
      int count = listeners.size();
      for (int i = 0; i < count; i++) {
        ((ButtonListener) listeners.elementAt(i)).buttonPressed(e);
      }
    }
  }

  public void mouseClicked(MouseEvent e) {
    fireButtonPressed(new ButtonEvent(this,text));
  }
  public void mousePressed(MouseEvent e) {
  }
  public void mouseReleased(MouseEvent e) {
  }
  public void mouseEntered(MouseEvent e) {
  }
  public void mouseExited(MouseEvent e) {
  }

  public void paint(Graphics g) {
    Color orig = g.getColor();
    g.setColor(textColor);
    g.drawString(text,10,15);
    g.setColor(getForeground());
    g.draw3DRect(0,0,getWidth()-1,getHeight()-1,true);
    g.setColor(orig);
  }

  public Dimension getMinimumSize() {
    return new Dimension( getFontMetrics(getFont()).stringWidth(text)+20, 30);
  }

  public Dimension getPreferredSize() { return getMinimumSize(); }
}