import java.awt.*;
import java.awt.event.*;
import java.applet.*;

/**
 * Description:
 * @author Mika Vesterholm
 * @version 1.0
 */

public class ChatApplet extends Applet {
  ClientProtocol protocol = new ClientProtocol();

  boolean isStandalone = false;
  TextArea textArea1 = new TextArea();
  Panel panel1 = new Panel();
  BorderLayout borderLayout1 = new BorderLayout();
  TextField textField1 = new TextField();
  Button button1 = new Button();
  GridBagLayout gridBagLayout1 = new GridBagLayout();
  /**Get a parameter value*/
  public String getParameter(String key, String def) {
    return isStandalone ? System.getProperty(key, def) :
      (getParameter(key) != null ? getParameter(key) : def);
  }

  /**Construct the applet*/
  public ChatApplet() {
  }
  /**Initialize the applet*/
  public void init() {
    try {
      jbInit();
    } catch(Exception e) {
      e.printStackTrace();
    }
  }
  /**Component initialization*/
  private void jbInit() throws Exception {
    this.setLayout(borderLayout1);
    button1.setLabel("Send");
    button1.addActionListener(new ChatApplet_button1_actionAdapter(this));
    panel1.setLayout(gridBagLayout1);
    protocol.addMessageListener(new ChatApplet_protocol_messageAdapter(this));
    textArea1.setEditable(false);
    this.add(textArea1, BorderLayout.CENTER);
    this.add(panel1, BorderLayout.SOUTH);
    panel1.add(textField1, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
    panel1.add(button1, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
  }
  /**Destroy the applet*/
  public void destroy() {
    try {
      protocol.quit();
    } catch (Exception ex) {
    }
  }

  /**Main method*/
  public static void main(String[] args) {
    final ChatApplet applet = new ChatApplet();
    applet.isStandalone = true;
    Frame frame;
    frame = new Frame() {
      protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
          applet.destroy();
          System.exit(0);
        }
      }
      public synchronized void setTitle(String title) {
        super.setTitle(title);
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      }
    };
    frame.setTitle("Applet Frame");
    frame.add(applet, BorderLayout.CENTER);
    applet.init();
    applet.start();
    frame.setSize(400,420);
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    frame.setLocation((d.width - frame.getSize().width) / 2, (d.height - frame.getSize().height) / 2);
    frame.setVisible(true);
  }

  void button1_actionPerformed(ActionEvent e) {
    sendMessage();
  }

  protected void sendMessage() {
    try {
      if ( !protocol.isConnected() ) {
        LoginDialog ld = new LoginDialog(new Frame(),"Login", true);
        ld.show();
        if (ld.getUserName() == null) return;
        protocol.connect("localhost",8989,ld.getUserName());
      }
      protocol.sendMessage(textField1.getText());
      textArea1.append(textField1.getText()+"\n");
      textField1.setText("");
      textField1.requestFocus();
    } catch (Exception e) {
      getToolkit().beep();
      if (isStandalone)
        System.out.println(e.getMessage());
      else
        showStatus(e.getMessage());
    }
  }

  void protocol_messageArrived(MessageEvent e) {
    String sender = e.getSender();
    if (e.getSender() == null) sender = "ERROR";
    textArea1.append(sender+": "+e.getMessage()+"\n");
  }
}

class ChatApplet_button1_actionAdapter implements java.awt.event.ActionListener {
  ChatApplet adaptee;

  ChatApplet_button1_actionAdapter(ChatApplet adaptee) {
    this.adaptee = adaptee;
  }
  public void actionPerformed(ActionEvent e) {
    adaptee.button1_actionPerformed(e);
  }
}

class ChatApplet_protocol_messageAdapter implements MessageListener {
  ChatApplet adaptee;

  ChatApplet_protocol_messageAdapter(ChatApplet adaptee) {
    this.adaptee = adaptee;
  }
  public void messageArrived(MessageEvent e) {
    adaptee.protocol_messageArrived(e);
  }
}
