/**
 * Title:
 * Description:
 * Copyright:
 * Company:
 * @author Mika Vesterholm
 * @version 1.0
 */
import java.awt.event.*;
import java.util.*;
import java.io.*;
import java.net.*;

public class ClientProtocol implements Runnable {
  Socket socket;
  BufferedWriter writer;
  BufferedReader reader;
  boolean connected = false;
  private transient Vector messageListeners = new Vector();

  /**
   * Connects to server and starts message receiver thread.
   * @param addr server address.
   * @param port server port.
   * @param username user name.
   * @throws Exception if fails to connect
   */
  public void connect(String addr, int port, String username) throws Exception {
    try {
      socket = new Socket(addr, port);
      writer = new BufferedWriter(new OutputStreamWriter(socket.getOutputStream(),"UTF-8"));
      reader = new BufferedReader(new InputStreamReader(socket.getInputStream(),"UTF-8"));
      //Protokollan mukainen yhteydenotto
      writer.write("ChatClient ver1.0\n");
      writer.write(username + "\n");
      writer.flush();
      if ( !"OK".equals(reader.readLine()) ) {
        disconnect();
        throw new Exception("User not accepted.");
      }
      connected = true;
      new Thread(this).start();
    } catch (IOException e) {
      e.printStackTrace();
      throw new Exception("Failed to connect to server!");
    }
  }

  /**
   * Sends message to server.
   * @throws Exception if not connected or send fails.
   */
  public void sendMessage(String message) throws Exception {
    if (!connected) throw new Exception("Not connected!");

    try {
      writer.write("Message\n");
      writer.write(message+"\n");
      writer.flush();
    } catch (Exception e) {
      disconnect();
      e.printStackTrace();
      throw new Exception("Failed to send message!");
    }
  }

  /**
   * Sends Quit message to server.
   * @throws Exception if not connected or send fails.
   */
  public void quit() throws Exception {
    if (!connected) throw new Exception("Not connected!");

    try {
      writer.write("Exit\n");
      writer.flush();
    } catch (Exception e) {
      disconnect();
      e.printStackTrace();
      throw new Exception("Failed to send exit!");
    }
  }

  /**
   * Closes streams.
   */
  protected void disconnect() {
    try {
      connected = false;
      if (reader != null) reader.close();
      if (writer != null) writer.close();
      if (socket != null) socket.close();
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
  }

  public boolean isConnected() { return connected; }

  public void run() {
    while(connected) {
      try {
        String line = reader.readLine();
        if ( "Message".equals(line) ) {
          String sender = reader.readLine();
          String message = reader.readLine();
          if (sender == null || message == null) {
            causeEvent(new MessageEvent(this,null,"Connection lost."));
            disconnect();
          } else {
            causeEvent(new MessageEvent(this,sender,message));
          }
        } else {
          causeEvent(new MessageEvent(this,null,"Invalid response."));
          disconnect();
        }
      } catch (IOException ioe) {
        if (connected) {
          ioe.printStackTrace();
          causeEvent(new MessageEvent(this,null,"Connection lost."));
          disconnect();
        }
      }
    }
  }


  private void causeEvent(MessageEvent e) {
    synchronized(messageListeners) {
      int size = messageListeners.size();
      for(int i=0;i<size;i++) {
        MessageListener l =
          (MessageListener)messageListeners.elementAt(i);
        l.messageArrived(e);
      }
    }
  }

  public void removeMessageListener(MessageListener l) {
    messageListeners.removeElement(l);
  }

  public void addMessageListener(MessageListener l) {
    messageListeners.addElement(l);
  }
}