/**
 * Title:
 * Description:
 * Copyright:
 * Company:
 * @author Mika Vesterholm
 * @version 1.0
 */
import java.sql.*;

public class LuoKanta {
  Connection con;

  public LuoKanta() throws Exception {
    //Ladataan tietokannan ajurit
    try {
      Class.forName("interbase.interclient.Driver");
    } catch (ClassNotFoundException cnfe) {
      cnfe.printStackTrace();
      throw new Exception("JDBC-ajurin rekisterinti eponnistui.");
    }
    try {
      String url = "jdbc:interbase://localhost/c:/databases/javakanta.gdb";
      con = DriverManager.getConnection(url,"sysdba","sala");
    } catch(SQLException se) {
      se.printStackTrace();
      throw new Exception("Kantaan ei saatu yhteytt.");
    }
  }

  void luoTaulut() throws Exception {
    try {
      String sql;
      Statement stmt = con.createStatement();

      sql = "CREATE TABLE TUOTTEET (" +
            " TID   INTEGER     NOT NULL PRIMARY KEY," +
            " NIMI  VARCHAR(40) NOT NULL," +
            " HINTA FLOAT," +
            " LKM   INTEGER     DEFAULT 0)";
      stmt.executeUpdate(sql);

      sql = "CREATE TABLE ASIAKKAAT (" +
            " ANO   INTEGER     NOT NULL PRIMARY KEY," +
            " ENIMI VARCHAR(40) NOT NULL," +
            " SNIMI VARCHAR(40) NOT NULL)";
      stmt.executeUpdate(sql);

      sql = "CREATE TABLE TILAUKSET(" +
            " TNO   INTEGER     NOT NULL PRIMARY KEY," +
            " ANO   INTEGER     NOT NULL," +
            " PVM   TIMESTAMP   NOT NULL)";
      stmt.executeUpdate(sql);

      sql = "CREATE TABLE TRIVI (" +
            " TNO   INTEGER     NOT NULL," +
            " TID   INTEGER     NOT NULL," +
            " LKM   INTEGER     NOT NULL," +
            " HINTA FLOAT)";
      stmt.executeUpdate(sql);

      stmt.close();
    } catch(SQLException se) {
      se.printStackTrace();
      throw new Exception("Taulujen luonti eponnistui.");
    }
  }

  void lisTietoa() throws Exception {
    try {
      String sql;
      Statement stmt = con.createStatement();

      sql = "INSERT INTO TUOTTEET " +
            "VALUES(1, 'Korppu', 3.5, 10)";
      stmt.executeUpdate(sql);
      sql = "INSERT INTO TUOTTEET " +
            "VALUES(2, 'Romppu', 7.5, 100)";
      stmt.executeUpdate(sql);
      sql = "INSERT INTO TUOTTEET " +
            "VALUES(3, 'Hiiri', 150, 5)";
      stmt.executeUpdate(sql);

      sql = "INSERT INTO ASIAKKAAT " +
            "VALUES(1, 'Matti', 'Meiklinen')";
      stmt.executeUpdate(sql);

      sql = "INSERT INTO ASIAKKAAT " +
            "VALUES(2, 'Aito', 'Keinotekoinen')";
      stmt.executeUpdate(sql);

      stmt.close();
    } catch(SQLException se) {
      se.printStackTrace();
      throw new Exception("Tiedon lisminen eponnistui.");
    }
  }

  void kysely() throws Exception {
    try {
      Statement stmt = con.createStatement();
      String sql = "SELECT * FROM TUOTTEET";
      ResultSet rs = stmt.executeQuery(sql);
      while(rs.next()) {
        int id = rs.getInt("TID");
        String nimi = rs.getString("NIMI");
        float hinta = rs.getFloat(3);
        int lkm = rs.getInt(4);
        System.out.println(nimi+" "+id+" "+hinta+" "+lkm);
      }
      rs.close();

      sql = "SELECT ENIMI, SNIMI, PVM "+
                   "FROM TILAUKSET T, ASIAKKAAT A "+
                   "WHERE A.ANO = T.ANO";
      rs = stmt.executeQuery(sql);
      while(rs.next()) {
        System.out.println(rs.getString(1) + " "
                          +rs.getString(2) + " "
                          +rs.getDate(3));
      }
      rs.close();

      stmt.close();
    } catch(SQLException se) {
      se.printStackTrace();
      throw new Exception("Taulujen haku eponnistui.");
    }
  }

  void valmisteltuKysely() throws Exception {
    try {
      PreparedStatement pivitLKM =
        con.prepareStatement("UPDATE TUOTTEET SET LKM = ? WHERE TID = ?");
      pivitLKM.setInt(1, 9); //Ensimminen kysymysmerkki
      pivitLKM.setInt(2, 1); //toinen kysymysmerkki
      pivitLKM.executeUpdate();

      PreparedStatement haeAsiakas =
        con.prepareStatement("SELECT * FROM ASIAKKAAT WHERE SNIMI = ?");
      haeAsiakas.setString(1, "Meiklinen");
      ResultSet rs = haeAsiakas.executeQuery();
    } catch(SQLException se) {
      se.printStackTrace();
      throw new Exception("Taulujen haku eponnistui.");
    }
  }

  void eponnistuvaTransaktio() {
    try {
      //Luodaan kaksi yhteytt, jotka esittvt eri asiakkaita
      String url = "jdbc:interbase://localhost/c:/databases/javakanta.gdb";
      Connection con1 = DriverManager.getConnection(url,"sysdba","sala");
      Connection con2 = DriverManager.getConnection(url,"sysdba","sala");
      //automaattinen pivitys pois kytst
      //con1.setAutoCommit(false);
      //con2.setAutoCommit(false);

      Statement stmt1 = con1.createStatement();
      Statement stmt2 = con2.createStatement();
      ResultSet rs1 = stmt1.executeQuery("SELECT LKM FROM TUOTTEET WHERE TID = 1");
      ResultSet rs2 = stmt2.executeQuery("SELECT LKM FROM TUOTTEET WHERE TID = 1");
      rs1.next(); rs2.next();
      int lkm1 = rs1.getInt("LKM");
      int lkm2 = rs2.getInt("LKM");
      System.out.println("Lukumr1: "+lkm1);
      System.out.println("Lukumr2: "+lkm2);
      lkm1-=5;
      lkm2-=4;

      stmt1.executeUpdate("UPDATE TUOTTEET SET LKM = "+lkm1+" WHERE TID = 1");
      //ensimminen asiakas lopettaa transaktion
      con1.commit();

      System.out.println("Yritetn toista muutosta");
      //Seuraava lause aiheuttaa poikkeuksen, koska arvoa on jo ehditty muuttaa
      stmt2.executeUpdate("UPDATE TUOTTEET SET LKM = "+lkm2+" WHERE TID = 1");
      con2.commit();
    } catch(SQLException se) {
      //se.printStackTrace();
      System.out.println("Odotettu poikkeus!");
    }
  }


void lisTilaus(int ANO, int TID[], int LKM[]) throws Exception{
  PreparedStatement lisaaTilaus = null;
  PreparedStatement lisaaTRivi = null;
  PreparedStatement kysyLKMHinta = null;
  PreparedStatement asetaLKM = null;
  //Lismist voisi yritt muutaman kerran silt varalta,
  //ett virheen on aiheuttanut pllekkiset transaktiot
  try {
    con.setAutoCommit(false);
    lisaaTilaus = con.prepareStatement("INSERT INTO TILAUKSET VALUES(?, ?, ?)");
    lisaaTRivi = con.prepareStatement("INSERT INTO TRIVI VALUES(?, ?, ?, ?)");
    kysyLKMHinta = con.prepareStatement("SELECT HINTA, LKM FROM TUOTTEET WHERE TID = ?");
    asetaLKM = con.prepareStatement("UPDATE TUOTTEET set LKM = ? WHERE TID = ?");
    //tilausnumeron genetointi - toivottavasti kukaan muu ei
    //satu saamaan samaa numeroa, silloin tilauksen lisys eponnistuu
    //InterBasessa olisi tarjolla mys laskuriin tarvittavat generaattorit,
    //mutta ne eivt ole SQL:n standardin mukaisia.
    int tno = (int)System.currentTimeMillis();

    lisaaTilaus.setInt(1,tno);
    lisaaTilaus.setInt(2,ANO);
    lisaaTilaus.setDate(3, new java.sql.Date(System.currentTimeMillis()));
    lisaaTilaus.executeUpdate();

    for (int i=0; i<TID.length; i++) {
      kysyLKMHinta.setInt(1,TID[i]);
      ResultSet rs = kysyLKMHinta.executeQuery();
      rs.next(); //Poikkeus, jos tuotetta ei lytynyt

      int varastossa = rs.getInt(2);
      if ( varastossa < LKM[i] ) {
        throw new Exception("Tuotetta "+TID[i]+" ei ole tarpeeksi varastossa!");
      }

      lisaaTRivi.setInt(1,tno);
      lisaaTRivi.setInt(2,TID[i]);
      lisaaTRivi.setInt(3,LKM[i]);
      lisaaTRivi.setFloat(4,rs.getFloat(1));
      lisaaTRivi.executeUpdate();
      rs.close();

      asetaLKM.setInt(1,varastossa-LKM[i]);
      asetaLKM.setInt(2, TID[i]);
      asetaLKM.executeUpdate();
    }
    con.commit();
  } catch(SQLException e) {
    con.rollback();
    throw e;
  } finally {
    con.setAutoCommit(true);
    if (lisaaTilaus != null) lisaaTilaus.close();
    if (lisaaTRivi != null) lisaaTRivi.close();
    if (kysyLKMHinta != null) kysyLKMHinta.close();
    if (asetaLKM != null) asetaLKM.close();
  }
}

  public static void main(String[] args) {
    try {
      LuoKanta lk = new LuoKanta();
      //lk.luoTaulut();
      //lk.lisTietoa();
      lk.kysely();
      //lk.valmisteltuKysely();
      //lk.eponnistuvaTransaktio();
      int tuotteet[] = {1,2};
      int mrt[]   = {2,1};
      //lk.lisTilaus(1,tuotteet,mrt);
    } catch (Exception e) {
      System.out.println(e.getMessage());
    }
  }

}