import java.awt.*;               // ScrollPane, PopupMenu, MenuShortcut, jne.
import java.awt.datatransfer.*;  // Clipboard, Transferable, DataFlavor, jne.
import java.awt.event.*;         // Uusi tapahtumaksittelymalli.
import java.io.*;                // Olioiden serialisointivirrat.
import java.util.zip.*;          // Tiedon pakkaus- ja purkuvirrat.
import java.util.Vector;         // Piirustus laitetaan vektoriin.
import java.util.Properties;     // Tulostusasetukset laitetaan Properties-olioon
/**
 * Tss luokassa sijoitetaan Scribble-komponentti ScrollPane-silin, 
 * ja Scrollpane-sili ikkunaan, johon listn yksinkertainen 
 * alasvetovalikko. Valikon valinnat voi valita pikanppimill. Tapahtumat  
 * ksitelln anonyymeill luokilla.
*/
public class ScribbleFrame extends Frame {
  /** Hyvin yksinkertainen main()-metodi ohjelmaa varten. */
  public static void main(String[] args) { new ScribbleFrame(); }
  /** Auki olevien ikkunoiden mr tytyy muistaa, jotta osataan lopettaa 
   *  kun viimeinen ikkunoista suljetaan.
   */
  protected static int num_windows = 0;
  /** Luodaan kehysikkuna, valikko ja ScrollPane-komponentti */
  public ScribbleFrame() {
    super("ScribbleFrame");                  // Luodaan ikkuna.
    num_windows++;                           // Kasvatetaan ikkunalaskuria.
    ScrollPane pane = new ScrollPane();      // Luodaan ScrollPane.
    pane.setSize(300, 300);                  // Mritelln sen koko.
    this.add(pane, "Center");                // Listn se kehykseen.
    Scribble scribble;
    scribble = new Scribble(this, 500, 500); // Suurempi piirustusalue.
    pane.add(scribble);                      // Listn ScrollPane-silin.
    MenuBar menubar = new MenuBar();         // Luodaan valikkorivi.
    this.setMenuBar(menubar);                // Listn se kehykseen.
    Menu file = new Menu("File");            // Luodaan File-valikko.
    menubar.add(file);                       // Listn se valikkoriville.
    // Luodaan kolme valikkovalintaa, joihin liittyy pikanppimet, ja 
    // listn ne valikkoon.
    MenuItem n, c, q;
    file.add(n=new MenuItem("New Window", new MenuShortcut(KeyEvent.VK_N)));
    file.add(c=new MenuItem("Close Window",new MenuShortcut(KeyEvent.VK_W)));
    file.addSeparator();                     // Laitetaan valikkoon erotin
    file.add(q = new MenuItem("Quit", new MenuShortcut(KeyEvent.VK_Q)));
    // Luodaan ja rekisteridn tapahtumakuunteluoliot 
    // valikkovalintoja varten.
    n.addActionListener(new ActionListener() {     // Avaa uuden ikkunan.
      public void actionPerformed(ActionEvent e) { new ScribbleFrame(); }
    });
    c.addActionListener(new ActionListener() {     // Sulkee tmn ikkunan.
      public void actionPerformed(ActionEvent e) { close(); }
    });
    q.addActionListener(new ActionListener() {     // Sulkee ohjelman.
      public void actionPerformed(ActionEvent e) { System.exit(0); }
    });
    // Toinen tapahtumakuuntelija. Tm kuuntelee ikkunan sulkupyyntj.
    this.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) { close(); }
    });
        // Asetetaan ikkunan koko ja tuodaan se nkyville.
    this.pack();
    this.show();
  }
  /** Suljetaan ikkuna. Jos tm on viimeinen auki oleva ikkuna, lopetetaan   
   */
  void close() {
    if (--num_windows == 0) System.exit(0);
    else this.dispose();
  }
}
/**
 * Tm luokka on itse tehty komponentti, johon voi piirt. Komponentissa on 
 * mys pikavalikko, jonka kautta voi valita piirustusvrin ja jota kautta 
 * psee ksiksi tulostus, leikkaa-liit ja tiedoston avaamis- sek 
 * talletustoimintoihin. Huomaa, ett luokka periytyy Component-luokastas 
 * eik Canvas-luokasta, joten se on resurssien kytn kannalta kevyt.
 */
class Scribble extends Component implements ActionListener {
  protected short last_x, last_y;                // Viimeisen napautuksen 
                                                 // koordinaatit.
  protected Vector lines = new Vector(256,256);  // Thn laitetaan talteen 
                                                 // piirustus.
  protected Color current_color = Color.black;   // Parhaillaan kytettv 
                                                 // vri.
  protected int width, height;                   // Toivottu koko.
  protected PopupMenu popup;                     // Pikavalikko.
  protected Frame frame;                         // Kehysikkuna, jossa 
                                                 // komponentti on.
  /** Tlle konstruktorille pit antaa Frame-ikkuna ja koko */
  public Scribble(Frame frame, int width, int height) {
    this.frame = frame;
    this.width = width;
    this.height = height;
    // Piirteleminen hoidetaan matalan tason tapahtumilla, joten tytyy
    // mritell, mist tapahtumista ollaan kiinnostuneita.
    this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
    this.enableEvents(AWTEvent.MOUSE_MOTION_EVENT_MASK);
    // Luodaan pikavalikko silmukassa. Huomaa, miten valikon 
    // komentomerkkijono erotetaan valikon otsikosta. Tm on hyv asia 
    // kansainvlistmist ajatellen.
    String[] labels = new String[] {
      "Clear", "Print", "Save", "Load", "Cut", "Copy", "Paste" };
    String[] commands = new String[] {
      "clear", "print", "save", "load", "cut", "copy", "paste" };
    popup = new PopupMenu();                   // Luodaan valikko
    for(int i = 0; i < labels.length; i++) {
      MenuItem mi = new MenuItem(labels[i]);   // Luodaan valikon valinta.
      mi.setActionCommand(commands[i]);        // Asetetaan sen 
                                               // komentomerkkijono.
      mi.addActionListener(this);              // Ja tapahtumakuuntelija.
      popup.add(mi);                           // Listn valinta 
                                               // pikavalikkoon.
    }
    Menu colors = new Menu("Color");           // Luodaan alivalikko.
    popup.add(colors);                         // Ja listn se 
                                               // pikavalikkoon.
    String[] colornames = new String[] { "Black", "Red", "Green", "Blue"};
    for(int i = 0; i < colornames.length; i++) {
      MenuItem mi = new MenuItem(colornames[i]);  // Luodaan alivalikon 
      mi.setActionCommand(colornames[i]);         // valinnat samalla 
      mi.addActionListener(this);                 // tavalla.
      colors.add(mi);
    }
    // Lopuksi, rekisteridn pikavalikko sille komponentille, jonka 
    // kohdalla sen pit nky.
    this.add(popup);
  }
  /** Kertoo kuinka suuri komponentista pitisi tehd. Metodi palauttaa aina
   *  Scribble()-konstruktorille vlitetyn koon */
  public Dimension getPreferredSize() { return new Dimension(width, height);    
  }
  /** Tm on ActionListener-metodi, jota pikavalikon valinnat kutsuvat */
  public void actionPerformed(ActionEvent event) {
    // Kysytn tapahtumaa vastaava komentomerkkijono ja kutsutaan sen 
    // perusteella metodia. Tss metodissa kutsutaan useita tss luokassa 
    // olevia mielenkiintoisia metodeita.
    String command = event.getActionCommand();
    if (command.equals("clear")) clear();
    else if (command.equals("print")) print();
    else if (command.equals("save")) save();
    else if (command.equals("load")) load();
    else if (command.equals("cut")) cut();
    else if (command.equals("copy")) copy();
    else if (command.equals("paste")) paste();
    else if (command.equals("Black")) current_color = Color.black;
    else if (command.equals("Red")) current_color = Color.red;
    else if (command.equals("Green")) current_color = Color.green;
    else if (command.equals("Blue")) current_color = Color.blue;
  }
  /** Piirretn kaikki piirustukseen kuuluvat viivat oikeilla vreill */
  public void paint(Graphics g) {
    for(int i = 0; i < lines.size(); i++) {
      Line l = (Line)lines.elementAt(i);
      g.setColor(l.color);
      g.drawLine(l.x1, l.y1, l.x2, l.y2);
    }
  }
  /**
   * Tm on matalan tason tapahtumaksittelymetodi, jota kutsutaan aina kun
   * tapahtuu hiiritapahtuma, johon ei liity hiiren liikett. Huomaa, kuinka
   * isPopupTrigger()-metodilla tarkistetaan, onko kyse ajoympristss 
   * kytettvst pikavalikon kynnistystoiminnosta. Pikavalikko tuodaan 
   * nkyviin show()-metodilla. Jos valikkoa ei nytet, metodi laittaa  
   * talteen hiiren painalluksen koordinaatit tai kutsuu yliluokan metodia.
   */
  public void processMouseEvent(MouseEvent e) {
    if (e.isPopupTrigger())                 // Jos pikavalikon kynnistys,
      popup.show(this, e.getX(), e.getY()); // nytetn valikko.
    else if (e.getID() == MouseEvent.MOUSE_PRESSED) {
      last_x = (short)e.getX(); last_y = (short)e.getY(); // Laitetaan 
                                                          // positio talteen.
    }
    else super.processMouseEvent(e);  // Reititetn muunlaiset 
                                      // tapahtumat eteenpin.
  }
  /**
   * Tt metodia kutsutaan, kun hiiri liikkuu. Metodissa listn 
   * piirustukseen viiva joka tuodaan mys nytlle ja laitetaan talteen.
   */
  public void processMouseMotionEvent(MouseEvent e) {
    if (e.getID() == MouseEvent.MOUSE_DRAGGED) {
      Graphics g = getGraphics();                  // Olio, johon piirretn.
      g.setColor(current_color);                    // Asetetaan vri.
      g.drawLine(last_x, last_y, e.getX(), e.getY()); // Piirretn viiva
      lines.addElement(new Line(last_x, last_y,       // ja laitetaan se mys 
                               (short) e.getX(),      // talteen.
                               (short)e.getY(),
                               current_color));
      last_x = (short) e.getX();  // Pannaan hiiren koordinaatit talteen.
      last_y = (short) e.getY();
    }
    else super.processMouseMotionEvent(e);  // Trke!
  }
  /** Tyhjennetn piirustus. Tt kutsutaan pikavalikon kautta */
  void clear() {
    lines.removeAllElements();   // Poistetaan piirustus ja piirretn
    repaint();                   // kaikki uudestaan.
  }
  /** Tulostetaan piirustus. Tt kutsutaan pikavalikon kautta */
  void print() {
    // Pyydetn PrintJob-olio. Olio tuo nkyviin tulostusdialogin.
    // Tulostusasetukset laitetaan talteen printprefs-olioon (joka 
    // luodaan alla).
    Toolkit toolkit = this.getToolkit();
    PrintJob job = toolkit.getPrintJob(frame, "Scribble", printprefs);
    // Jos kyttj peruutti tulostuksen tulostusdialogissa, ei tehd mitn.
    if (job == null) return;
    // Pyydetn Graphics-olio ensimmist sivua varten.
    Graphics page = job.getGraphics();
    // Kysytn Scribble-komponentin ja sivun koko.
    Dimension size = this.getSize();
    Dimension pagesize = job.getPageDimension();
    // Keskitetn tulostus keskelle sivua. Muuten se tulisi sivun vasempaan 
    // ylreunaan.
    page.translate((pagesize.width - size.width)/2,
                   (pagesize.height - size.height)/2);
    // Piirretn tulostusalueen ymprille kehys, jotta kuva nyttisi 
    // paremmalta.
    page.drawRect(-1, -1, size.width+1, size.height+1);
    // Asetetaan tulostuksen rajausalue, jotta piirustus ei menisi rajojen 
    // yli. Nytlle piirrettess rajaaminen tapahtuu automaattisesti mutta 
    // ei paperille tulostettaessa.
    page.setClip(0, 0, size.width, size.height);
    // Tulostetaan tm Scribble-komponentti. Oletusarvoisesti kutsutaan vain 
    // paint()-metodia. 
    this.print(page);
    // Lopetetaan tulostaminen.
    page.dispose();   // Lopetetaan sivu  lhetetn se kirjoittimelle.
    job.end();        // Lopetetaan tulostusty.
  }
  /** Thn Properties-olioon laitetaan talteen tulostusdialogissa tehdyt
   *  tulostusasetukset.
   */
  private static Properties printprefs = new Properties();
  /**
   * Tm on DataFlavor-olio, joka kuvastaa tss ohjelmassa leikkaa-liit-
   * tyyppisesti siirrettv tietoa. Tieto siirretn serialisoituna Vector-
   * oliona. Huomaa, ett Java 1.1.1-versiossa tm toimii ohjelman sisll,
   * mutta ei ohjelmien vlill. Java 1.1.1-versiolla ohjelmien vlill 
   * siirrettvtieto on rajoitettu etukteen mriteltyihin merkkijono- ja    
   * tekstityyppeihin.
   */
  public static final DataFlavor dataFlavor =
      new DataFlavor(Vector.class, "ScribbleVectorOfLines");
  /**
   * Kopioidaan piirustus ja laitetaan se talteen SimpleSelection-olioon 
   * (joka on mritelty alla). Viedn tm olio sitten leikepydlle 
   * liittmist varten.
   */
  public void copy() {
    // Pyydetn jrjestelmn leikepyt.
    Clipboard c = this.getToolkit().getSystemClipboard();
    // Kopioidaan ja talletetaan piirustus Transferable-olioon.
    SimpleSelection s = new SimpleSelection(lines.clone(), dataFlavor);
    // Laitetaan olio leikepydlle.
    c.setContents(s, s);
  }
  /** Leikkaa toimii samalla tavalla kuin kopoiointi, paitsi ett piirustus
   *  tyhjennetn sen jlkeen.
   */
  public void cut() { copy(); clear();  }
  /**
   * Pyydetn jrjestelmn leikepydlt Transferable-olion sislt ja 
   * pyydetn saadulta oliolta vastaamansa piirustuksen sislt. Jos 
   * jompikumpi toimenpiteist eponnistuu, annetaan nimerkki.
   */
  public void paste() {
    Clipboard c = this.getToolkit().getSystemClipboard();  // Pyydetn 
                                                           // leikepyt.
    Transferable t = c.getContents(this);                  // Pyydetn sen 
                                                           // sislt.
    if (t == null) {   // Ellei ole liitettvi tietoja, nimerkki.
      this.getToolkit().beep();
      return;
    }
    try {
      // Pyydetn, ett leikepydn sislt muunnetaan tss tuettuun 
      // muotoon. Muunnoksesta aiheutuu poikkeus, mikli se ei onnistu.
      Vector newlines = (Vector) t.getTransferData(dataFlavor);
      // Listn kaikki liitetyt viivat kuvaan.
      for(int i = 0; i < newlines.size(); i++)
        lines.addElement(newlines.elementAt(i));
      // Ja piirretn koko kuva uudestaan.
      repaint();
    }
    catch (UnsupportedFlavorException e) {
      this.getToolkit().beep();   // Jos leikepydll on muun tyyppist tietoa
    }
    catch (Exception e) {
      this.getToolkit().beep();   // Jos jokin muu menee pieleen...
    }
  }
  /**
   * Tss sisemmss luokassa on toteutettu Transferable- ja ClipboardOwner-
   * rajapinnat, joita tarvitaan, kun siirretn tietoja. Luokka on 
   * yksinkertainen ja se lhinn vain muistaa valitun olion ja antaa sen 
   * tarjolle jossakin mritellyist muodoista.
   */
     static class SimpleSelection implements Transferable, ClipboardOwner {
    protected Object selection;    // Siirrettvt tiedot.
    protected DataFlavor flavor;   // Ainoa tuettu esitysmuoto
    public SimpleSelection(Object selection, DataFlavor flavor) {
      this.selection = selection;  // Mritelln tiedot.
      this.flavor = flavor;        // Mritelln muoto.
    }
    /** Palautetaan luettelo tuetuista muodoista. Tss tapauksessa vain yksi 
     *  muoto
     */
    public DataFlavor[] getTransferDataFlavors() {
      return new DataFlavor[] { flavor };
    }
    /** Tarkistetaan, tuetaanko kysytty esitysmuotoa */
    public boolean isDataFlavorSupported(DataFlavor f) {
      return f.equals(flavor);
    }
    /** Jos esitysmuoto sopii, siirretn tiedot (eli palautetaan ne) */
    public Object getTransferData(DataFlavor f)
         throws UnsupportedFlavorException {
      if (f.equals(flavor)) return selection;
      else throw new UnsupportedFlavorException(f);
    }
    /** Tm on ClipboardOwner-metodi. Sit kutsutaan, kun tiedot eivt en
     *  ole leikepydll. Tilanteessa ei tarvitse tehd juuri mitn. */
    public void lostOwnership(Clipboard c, Transferable t) {
      selection = null;
    }
  }
  /**
   * Kysytn kyttjlt tiedostonimi, talletetaan piirustus kyseiseen 
   * tiedostoon. Serialisoidaan viivat sisltv vektori ObjectOutputStream-
   * oliolla. Serialisoidut oliot pakataan GZIPOutputStream-oliolla. Pakattu 
   * ja serialisoitu data kirjoitetaan tiedostoon FileOutputStream-oliolla. 
   * Ei pid unohtaa tyhjent ja sulkea virtaa.
   */
  public void save() {
    // Luodaan tiedostodialogi, jolla kysytn kyttjlt tiedostonimi.
    FileDialog f = new FileDialog(frame, "Save Scribble", FileDialog.SAVE);
    f.show();                        // Nytetn dialogi. Kutsusta palataan 
                                     // pois vasta kun dialogi kuitataan.
    String filename = f.getFile();   // Pyydetn kyttjn vastaus.
    if (filename != null) {          // Jos ei painettu Peruuta-nappia.
      try {
        // Luodaan piirustuksen talletuksessa tarvittavat virrat.
        // Talletetaan tiedostoon
        FileOutputStream fos = new FileOutputStream(filename); 
        GZIPOutputStream gzos = new GZIPOutputStream(fos);     // Pakattuna
        ObjectOutputStream out = new ObjectOutputStream(gzos); // Oliot
        out.writeObject(lines);      // Kirjoitetaan koko vektori.
        out.flush();                 // Tulos pit aina tyhjent.
        out.close();                 // Ja sulkea virta.
      }
      // Tulostetaan poikkeukset. Nm pitisi kyll nytt dialogissa...
      catch (IOException e) { System.out.println(e); }
    }
  }
  /**
   * Kysytn tiedostonimi ja ladataan piirustus kyseisest tiedostosta.
   * Luetaan pakattu, serialisoitu data FileInputStream-virralla.
   * Puretaan data GZIPInputStream-virralla. Deserialisoidaan viivavektori
   * ObjectInputStream-oliolla. Korvataan parhaillaan muistissa olevat
   * viivat ladatuilla viivoilla ja piirretn kuva uudestaan.
   */
  public void load() {
    // Luodaan tiedostodialogi, jolla kysytn kyttjlt tiedostonimi.
    FileDialog f = new FileDialog(frame, "Load Scribble", FileDialog.LOAD);
    f.show();                         // Nytetn dialogi.
    String filename = f.getFile();    // Kysytn kyttjn vastaus.
    if (filename != null) {           // Jos ei painettu peruutusnppint.
      try {
        // Luodaan tarvittavat virrat
        // Luetaan tiedostosta
        FileInputStream fis = new FileInputStream(filename); 
        GZIPInputStream gzis = new GZIPInputStream(fis);     // Puretaan
        ObjectInputStream in = new ObjectInputStream(gzis);  // Luetaan oliot
        // Luetaan olio sisn. Olion pitisi olla viivavektori.
        Vector newlines = (Vector)in.readObject();
        in.close();                    // Suljetaan virta.
        lines = newlines;              // Asetetaan viivat.
        repaint();                     // Ja nytetn piirustus uudestaan.
      }
      // Tulostetaan poikkeukset. Nm pitisi kyll nytt dialogissa...
      catch (Exception e) { System.out.println(e); }
    }
  }
  /** Luokka, joka vastaa piirustuksen yhden viivan koordinaatteja ja vri.
   *  Koko piirustus on talletettu vektoriin, joka sislt tllaisia olioita   
   */
  static class Line implements Serializable {
    public short x1, y1, x2, y2;
    public Color color;
    public Line(short x1, short y1, short x2, short y2, Color c) {
      this.x1 = x1; this.y1 = y1; this.x2 = x2; this.y2 = y2; this.color = c;
    }
  }
}
