package oreilly.beans.yesno;      // Laitetaan tm komponentti omaan pakettiinsa.
import java.awt.*;
import java.awt.event.*;
import java.util.*;
public class YesNoDialog {
  // Komponentin ominaisuudet.
  protected String message, title;
  protected String yesLabel, noLabel, cancelLabel;
  protected int alignment;
  protected Font font = new Font("Serif", Font.PLAIN, 12);
  protected Color background = SystemColor.control;
  protected Color foreground = SystemColor.controlText;
  // Vakioita tasaustietoa varten.
  public static final int LEFT = MultiLineLabel.LEFT;
  public static final int RIGHT = MultiLineLabel.RIGHT;
  public static final int CENTER = MultiLineLabel.CENTER;
  // Metodit, joilla kysytn ominaisuuksien arvot.
  public String getMessage() { return message; }
  public String getTitle() { return title; }
  public String getYesLabel() { return yesLabel; }
  public String getNoLabel() { return noLabel; }
  public String getCancelLabel() { return cancelLabel; }
  public int getAlignment() { return alignment; }
  public Font getFont() { return font; }
  public Color getBackground() { return background; }
  public Color getForeground() { return foreground; }
  // Metodit, joilla asetetaan ominaisuuksien arvot.
  public void setMessage(String m) { message = m; }
  public void setTitle(String t) { title=t; }
  public void setYesLabel(String l) { yesLabel = l; }
  public void setNoLabel(String l) { noLabel = l; }
  public void setCancelLabel(String l) { cancelLabel = l; }
  public void setAlignment(int a) { alignment = a; }
  public void setFont(Font f) { font = f; }
  public void setBackground(Color bg) { background = bg; }
  public void setForeground(Color fg) { foreground = fg; }
  /** Tss kentss on joukko rekisterityneit ActionListener-olioita.
   *  Vector on sisisesti synkronoitu, joten sen kyttvuorosta ei tarvitse 
   *  kilpailla
   */
  protected Vector listeners = new Vector();
  /** Rekisteridn tapahtumakuuntelija, jolle kerrotaan, kun painetaan 
   *  nappia 
   */
  public void addAnswerListener(AnswerListener l) {
    listeners.addElement(l);
  }
  /** Poistetaan Answer-kuuntelija luettelosta */
  public void removeAnswerListener(AnswerListener l) {
    listeners.removeElement(l);
  }
  /** Lhetetn tapahtuma kaikille rekisterityneille kuuntelijoille */
  public void fireEvent(AnswerEvent e) {
    // Tehdn kopio listasta ja lhetetn sanoma listan perusteella.
    // Tst seuraa, ett alkuperisest listasta voi poistaa tai siihen
    // voi list kuuntelijoita vastauksena thn sanomaan. Vektori pitisi 
    // voida kyd lpi enumeraattorilla, mutta enumeraattori ei tee vektorista
    // sisist kopiotaan.
    Vector list = (Vector) listeners.clone();
    for(int i = 0; i < list.size(); i++) {
      AnswerListener listener = (AnswerListener)list.elementAt(i);
      switch(e.getID()) {
      case AnswerEvent.YES: listener.yes(e); break;
      case AnswerEvent.NO:  listener.no(e); break;
      case AnswerEvent.CANCEL: listener.cancel(e); break;
      }
    }
  }
  /** Argumentiton komponenttikonstruktori, jossa ominaisuuksille annetaan 
   *  oletusarvot
   */
  public YesNoDialog() {
    this("Question", "Your nMessage nHere", "Yes", "No", "Cancel", LEFT);
  }
  /** Ksin komponenttia kyttville ohjelmoijille tarkoitettu konstruktori 
   */
  public YesNoDialog(String title, String message,
                     String yesLabel, String noLabel, String cancelLabel,
                     int alignment) {
    this.title = title;
    this.message = message;
    this.yesLabel = yesLabel;
    this.noLabel = noLabel;
    this.cancelLabel = cancelLabel;
    this.alignment = alignment;
  }
  /** Tll metodilla komponentti saadaan nyttmn dialogi */
  public void display() {
    // Luodaan kehysikkuna, jossa on haluttu otsikko. Olisi mukavaa, jos
    // voisi kytt Dialog-oliota, mutta sellaiselle pit vlitt Frame-
    // argumentti, ja nytt silt, ett BDK:n beanbox-tykalu osaa kutsua
    // vain sellaisia metodeita, joille ei anneta argumentteja.
    final Frame frame = new Frame(title);
    // Mritelln ikkunan asettelun hallintaolio.
    frame.setLayout(new BorderLayout(15, 15));
    // Mrtn fontti ja vrit, jos sellaiset on annettu.
    if (font != null) frame.setFont(font);
    if (background != null) frame.setBackground(background);
    if (foreground != null) frame.setForeground(foreground);
    // Laitetaan ilmoitus keskelle ikkunaa.
    frame.add("Center", new MultiLineLabel(message, 20, 20, alignment));
    // Luodaan nappeja varten kuunteluoliot. Kun nappia painetaan, kuuntelija
    // sulkee ensin dialogin, mink jlkeen se luo AnswerEvent-olion, joka
    // vastaa sit nappia, jota painettiin. Tama tapahtumaolio lhetetn 
    // kaikille rekisterityneille kuuntelijoille aiemmin mritellyll
    // fireEvent()-metodilla.
    ActionListener listener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        frame.dispose();     // Piilotetaan ikkuna
        if (listeners != null) {      
          // Ilmoitetaan rekisterityneille kuuntelijoille
          String cmd = e.getActionCommand();
          int type;
          if (cmd.equals("yes")) type = AnswerEvent.YES;
          else if (cmd.equals("no")) type = AnswerEvent.NO;
          else type = AnswerEvent.CANCEL;
          fireEvent(new AnswerEvent(YesNoDialog.this, type));
        }
      }
    };
    // Luodaan paneeli dialogin nappeja varten ja laitetaan se dialogin 
    // alalaitaan.
    // Mritelln paneelia varten asettelunhallintaolio.
    Panel buttonbox = new Panel();
        buttonbox.setLayout(new FlowLayout(FlowLayout.CENTER, 25, 15));
    frame.add("South", buttonbox);
    // Luodaan jokainen mritelty nappi ja asetetaan samalla nappia vastaava
    // kuuntelijaolio ja komentomerkkijono. Napit listn buttonbox-
    // nappipaneeliin.
    if ((yesLabel != null) && (yesLabel.length() > 0)) {
      Button yes = new Button(yesLabel);      // Luodaan nappi.
      yes.setActionCommand("yes");            // Asetetaan komentomerkkijono.
      yes.addActionListener(listener);        // Asetetaan kuuntelija.
      buttonbox.add(yes);                     // Listn nappi paneeliin.
    }
    if ((noLabel != null) && (noLabel.length() > 0)) {
      Button no = new Button(noLabel);
      no.setActionCommand("no");
      no.addActionListener(listener);
      buttonbox.add(no);
    }
    if ((cancelLabel != null) && (cancelLabel.length() > 0)) {
      Button cancel = new Button(cancelLabel);
      cancel.setActionCommand("cancel");
      cancel.addActionListener(listener);
      buttonbox.add(cancel);
    }
    // Jrjestelln lopuksi dialogi sopivan kokoiseksi ja nytetn se.
    frame.pack();
    frame.show();
  }
  /**
   * main()-metodi, jolla esitelln tt luokkaa ja jonka avulla luokkaa voi 
   * testata.
   */
  public static void main(String[] args) {
    // Luodaan InfoDialog-olio mritellen samalla otsikko ja ilmoitus:
    YesNoDialog d =
      new YesNoDialog("YesNoDialog Test",
                      "There are unsaved files. n" +
                      "Do you want to save them before quitting?",
                      "Yes, save and quit",
                      "No, quit without saving",
                      "Cancel; dont quit",
                      YesNoDialog.CENTER);
    // Rekisteridn kuuntelija dialogille. Tm kuuntelija vain 
    // tulostaa ilmoituksen konsolille.
    d.addAnswerListener(new AnswerListener() {
      public void yes(AnswerEvent e) { System.out.println("Yes"); }
      public void no(AnswerEvent e) { System.out.println("No"); }
      public void cancel(AnswerEvent e) { System.out.println("Cancel"); }
    });
    // Nytetn dialogi. Dialogi poistaa itsens nytlt automaattisesti.
    d.display();
  }
}