import java.awt.*;
import java.awt.event.*;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;
/** Apuluokka, jolla voi luoda lokalisoituja valikoita */
public class SimpleMenu {
  /** Metodi, jolla valikko luodaan */
  public static Menu create(String bundlename,
                            String menuname, String[] itemnames,
                            ActionListener listener, boolean popup) {
    // Pyydetn valikkoa varten resurssinippu.
    ResourceBundle b = ResourceBundle.getBundle(bundlename);
    // Pyydetn nipusta valikon nimi. Kytetn nime oletusotsikkona.
    String menulabel;
    try { menulabel = b.getString(menuname + ".label"); }
    catch(MissingResourceException e) { menulabel = menuname; }
    // Luodaan valikko.
    Menu m;
    if (popup) m = new PopupMenu(menulabel);
    else m = new Menu(menulabel);
    // Kydn lpi kaikki valikkoon tulevat valinnat.
    for(int i = 0; i < itemnames.length; i++) {
      // Etsitn valinnan otsikko. Oletuksena kytetn valinnan nime.
      String itemlabel;
      try {itemlabel=b.getString(menuname + "." + itemnames[i] + ".label"); }
      catch (MissingResourceException e) { itemlabel = itemnames[i]; }
      // Etsitn valintaa vastaava pikanppin ja luodaan sellainen jos oli.
      String shortcut;
      try{shortcut = b.getString(menuname + "." + itemnames[i]+".shortcut"); }
      catch (MissingResourceException e) { shortcut = null; }
      MenuShortcut ms = null;
      if (shortcut != null) ms = new MenuShortcut(shortcut.charAt(0));
      // Luodaan valikon valinta.
      MenuItem mi;
      if (ms != null) mi = new MenuItem(itemlabel, ms);
      else mi = new MenuItem(itemlabel);
      // Rekisteridn tapahtumakuuntelija ja valintaa vastaava komento.
      if (listener != null) {
        mi.addActionListener(listener);
        mi.setActionCommand(itemnames[i]);
      }
      // Listn valinta valikkoon.
      m.add(mi);
    }
    // Palautetaan automaattisesti lokalisoitu valikko.
    return m;
  }
  /** Yksinkertainen testiohjelma yllolevaa koodia varten */
  public static void main(String[] args) {
    // Asetetaan oletuslokaali komentoriviargumentin perusteella.
    if (args.length == 2) Locale.setDefault(new Locale(args[0], args[1]));
    Frame f = new Frame("SimpleMenu Test");  // Luodaan ikkuna.
    MenuBar menubar = new MenuBar();         // Luodaan valikkorivi.
    f.setMenuBar(menubar);                   // Listn valikkorivi 
                                             // ikkunaan.
    // Luodaan valikko apumetodilla (oletuslokaalin mukainen).
    Menu colors = SimpleMenu.create("Menus", "colors",
                                    new String[] { "red", "green", "blue" },
                                    null, false);
    menubar.add(colors);                     // Listn valikko 
                                             // valikkoriville.
    f.setSize(300, 150);                     // Asetetaan ikkunan koko.
    f.show();                                // Nytetn ikkuna.
  }
}