import java.text.*;
import java.io.*;
import java.util.*;
/**
 * Apuluokka, jolla voi nytt lokalisoidun poikkeusilmoituksen, joka
 * riippuu poikkeuksen luokasta. Luokassa kytetn MessageFormat-luokkaa
 * ja sit kytettess voidaan nytt viisi merkkijonoa:
 *   {0}: Virheen tai poikkeuksen sisltm teksti.
 *   {1}: Poikkeuksen tai virheen tydellinen luokkanimi.
 *   {2}: Arvaus, miss tiedostossa poikkeus tapahtui.
 *   {3}: Ko. tiedoston rivinumero.
 *   {4}: Kuluva piv ja kellonaika.
 * Ilmoitustekstit etsitn resurssinipusta, jonka perusnimi on "Errors".
 * Resurssin nimen kytetn poikkeusolion luokan tydellist nime.
 * Jos tietty poikkeusta vastaavaa resurssia ei ole, kysytn yliluokilta.
 */
public class LocalizedError {
  public static void display(Throwable error) {
    ResourceBundle bundle;
    // Pyydetn resurssinippu.
    // Jos sit ei ole, nytetn ilmoitus lokalisoimatta.
    try { bundle = ResourceBundle.getBundle("Errors"); }
    catch (MissingResourceException e) {
      error.printStackTrace(System.err);
      return;
    }
    // Etsitn ko. nipusta resurssi virheen (tai sen yliluokan nimell)
    // Jos resurssia ei ole, nytetn ilmoitus lokalisoimatta.
    String message = null;
    Class c = error.getClass();
    while((message == null) && (c != Object.class)) {
      try { message = bundle.getString(c.getName()); }
      catch (MissingResourceException e) { c = c.getSuperclass(); }
    }
    if (message == null) { error.printStackTrace(System.err);  return; }
    // Yritetn ptell tiedosto ja rivi, jolla poikkeus tapahtui. 
    // Tulostetaan virheen pinovedos merkkijonoon ja tulkitaan merkkijono 
    // niin, ett ensimminen pinovedoksessa oleva rivinumero tulee 
    // ensimmisen tai toisen kaksoispisteen jlkeen. Oletetaan, ett tm 
    // kohta on ensimminen paikka, jota ohjelmoija itse voi hallita joten 
    // valitaan se paikaksi, jossa poikkeus tapahtui.
    String filename = "";
    int linenum = 0;
    try {
      // Merkkijonoon kirjoittava tulosvirta.
      StringWriter sw = new StringWriter();    
      PrintWriter out = new PrintWriter(sw);   // PrintWriter-olio.
      error.printStackTrace(out);              // Tulostetaan vedos pinosta.
      // Pyydetn vedos merkkijonona.
      String trace = sw.toString();            
      // Etsitn ensimminen kaksoispiste.
      int pos = trace.indexOf(':');            
      // Jos virheeseen liittyy teksti,etsitn seuraava kaksoispiste.
      if (error.getMessage() != null)          
        pos = trace.indexOf(':', pos+1);       
      int pos2 = trace.indexOf(')', pos);      // Etsitn rivinumeron loppu.
      // Otetaan rivinumero.
      linenum = Integer.parseInt(trace.substring(pos+1,pos2));  
      // Takaisin tiedostonimen alkuun.
      pos2 = trace.lastIndexOf('(', pos);      
      filename = trace.substring(pos2+1, pos); // Otetaan tiedostonimi.
    }
    // Poikkeukset jtetn ksittelemtt.
        catch (Exception e) { ; }                  
    // Luodaan taulukko, johon viedn ilmoituksen vaihtuvat osat.
    String errmsg = error.getMessage();
    Object[] args = {
      ((errmsg!= null)?errmsg:""), error.getClass().getName(),
      filename, new Integer(linenum), new Date()
    };
    // Nytetn lopuksi lokalisoitu virheilmoitus kytten
    // MessageFormat.format()-metodia, joka korvaa ilmoituksen vaihtuvat
    // osat.
    System.out.println(MessageFormat.format(message, args));
  }
}