import java.lang.reflect.*;
/** Ohjelma, joka tulostaa kuvauksen annetusta luokasta */
public class ShowClass {
  /** main-metodi. Tulostaa tietoja mritellyst luokasta */
  public static void main(String[] args) throws ClassNotFoundException {
    Class c = Class.forName(args[0]);
    print_class(c);
  }
  /** Nytetn luokan tai rajapinnan mreet, nimi, yliluokka ja rajapinnat.
   *  Sitten luetellaan kaikki konstruktorit, kentt ja metodit. */
  public static void print_class(Class c)
  {
    // Tulostetaan mreet, tyyppi (luokka vai rajapinta), nimi ja yliluokka.
    if (c.isInterface()) {
      // Mreisiin tulee mukaan "interface"-mre...
      System.out.print(Modifier.toString(c.getModifiers()) +  c.getName());
    }
    else
      System.out.print(Modifier.toString(c.getModifiers()) + " class " +
                       c.getName() +
                       " extends " + c.getSuperclass().getName());
    // Tulostetaan luokan tai rajapinnan rajapinnat ja sen 
    // perimt rajapinnat.
    Class[] interfaces = c.getInterfaces();
    if ((interfaces != null) && (interfaces.length > 0)) {
      if (c.isInterface()) System.out.println(" extends ");
      else System.out.print(" implements ");
      for(int i = 0; i < interfaces.length; i++) {
        if (i > 0) System.out.print(", ");
        System.out.print(interfaces[i].getName());
      }
    }
    System.out.println(" {");            // Aloitetaan jsenlistaus.
    // Etsitn ja tulostetaan luokan jsenet.
    System.out.println(" // Constructors");
    Constructor[] constructors = c.getDeclaredConstructors();
    // Tulostetaan konstruktorit.
    for(int i = 0; i < constructors.length; i++)      
      print_method_or_constructor(constructors[i]);
    System.out.println(" // Fields");
    Field[] fields = c.getDeclaredFields();           // Pyydetn kentt.
    for(int i = 0; i < fields.length; i++)            // Tulostetaan ne.
      print_field(fields[i]);
    System.out.println(" // Methods");
    Method[] methods = c.getDeclaredMethods();        // Pyydetn metodit.
    for(int i = 0; i < methods.length; i++)           // Tulostetaan ne.
      print_method_or_constructor(methods[i]);
    System.out.println("}");             // Luokan jsenten listauksen loppu.
  }
  /** Palauttaa rajapinnan tai perustietotyypin nimen. Osaa 
   *  ksitell taulukot 
   */
  public static String typename(Class t) {
    String brackets = "";
    while(t.isArray()) {
      brackets += "[]";
      t = t.getComponentType();
    }
    return t.getName() + brackets;
  }
  /** Palauttaa mreet merkkijonossa. Ottaa huomioon asettelun. */
  public static String modifiers(int m) {
    if (m == 0) return "";
    else return Modifier.toString(m) + " ";
  }
  /** Tulostetaan kentn mreet, tyyppi, ja nimi */
  public static void print_field(Field f) {
    System.out.println("  " +
                       modifiers(f.getModifiers()) +
                       typename(f.getType()) + " " + f.getName() + ";");
  }
  /** Tulostetaan metodin tai konstruktorin mreet, paluuarvon tyyppi, nimi
   *  parametrien tyypit ja poikkeukset. Huomaa, ett Member-rajapintaa 
   *  kyttmll tll metodilla voi ksitell sek Method- ett 
   *  Constructor-olioita. 
   */
  public static void print_method_or_constructor(Member member) {
    Class returntype=null, parameters[], exceptions[];
    if (member instanceof Method) {
      Method m = (Method) member;
      returntype = m.getReturnType();
      parameters = m.getParameterTypes();
      exceptions = m.getExceptionTypes();
    } else {
      Constructor c = (Constructor) member;
      parameters = c.getParameterTypes();
      exceptions = c.getExceptionTypes();
    }
    System.out.print("  " + modifiers(member.getModifiers()) +
                     ((returntype!=null)? typename(returntype)+" " : "") +
                     member.getName() + "(");
    for(int i = 0; i < parameters.length; i++) {
      if (i > 0) System.out.print(", ");
      System.out.print(typename(parameters[i]));
    }
    System.out.print(")");
    if (exceptions.length > 0) System.out.print(" throws ");
    for(int i = 0; i < exceptions.length; i++) {
          if (i > 0) System.out.print(", ");
      System.out.print(typename(exceptions[i]));
    }
    System.out.println(";");
  }
}