/************************************************************************************

vieraskirja.java

Servletti, joka toimii Web-pohjaisena Vieraskirjana.
Vieraskirjan tiedot sijaitsevat Access-tietokantataulussa.
Yhteys tietokantaan otetaan JDBC-ODBC-sillan kautta.

Jos servlettiin otetaan yhteys GET-metodilla, servletti palauttaa
vieraskirjan sislln, kytnnss vieraskirjassa olevien henkiliden
nimet ja viestit. Jos henkil haluaa nytt mys shkpostiosoitteensa, henkiln nimest
tehdn hyperlinkki hnen shkpostiosoitteeseensa.

Jos servlettiin otetaan yhteys POST-metodilla, tallennetaan lomakkeen tiedot tietokantaan.

Metodit:
	doGet()			- Ohjaa vieraskirjan toimintoja
	doPost()		- Siirt kutsun doGet()-metodille
	kirjoitaHTMLAlku()	- Kirjoittaa HTML-koodin pakolliset alkutagit
	kirjoitaHTMLLoppu()	- Kirjoittaa HTML-koodin pakolliset lopputagit
	kirjoitaVieraskirjaan()	- Lis tietokantaan uuden tietueen
	haeVieraskirja()	- Hakee Vieraskirjan sislln ja muotoilee siit HTML-sivun

Tekij: Pekka Malmirae, 12/1999

************************************************************************************/

package omatServletit;

import java.io.*;
import java.sql.*;
import javax.servlet.*;
import javax.servlet.http.*;

public class vieraskirja extends HttpServlet
{
  private String kentat[] = {"nimi", "email", "linkki", "lhiosoite", "postinumero", "postitoimipaikka", "puhelin", "viesti"};
  private String ajuri = "sun.jdbc.odbc.JdbcOdbcDriver";
  private String tietokanta = "jdbc:odbc:vieraskirja";
  private String taulu = "Vieraat"; 


  public void doGet(HttpServletRequest req, HttpServletResponse res) 
  	throws ServletException, IOException
  {
    res.setContentType("text/html");
    PrintWriter out = res.getWriter();
    
    kirjoitaHTMLAlku("Vieraskirja", out);
    
    if (req.getMethod().equals("GET")) haeVieraskirja(out);
    else kirjoitaVieraskirjaan(req, out);

    kirjoitaHTMLLoppu(out);
  }
  
  public void doPost(HttpServletRequest req, HttpServletResponse res)
  	throws ServletException, IOException
  {
  	doGet(req,res);
  }
  
  private void kirjoitaHTMLAlku(String otsikko, PrintWriter out)
  {
    out.println("<HTML>");
    out.println("<HEAD><TITLE>"+ otsikko +"</TITLE></HEAD>");
    out.println("<BODY>");
  }

  private void kirjoitaHTMLLoppu(PrintWriter out)
  {
    out.println("</BODY></HTML>");
  }
  

  private void kirjoitaVieraskirjaan(HttpServletRequest req, PrintWriter out)
  {
    String parametri;
    Connection yhteys = null;
    PreparedStatement lauseke = null;
    ResultSet tulos = null;

    try {
      Class.forName(ajuri).newInstance();
    } catch (Exception ee) { out.println("Virhe: " +ee.getMessage()); }  
    try {
      yhteys = DriverManager.getConnection(tietokanta);
      String kentta_jono = "";
      for (int i=0;i<kentat.length-1;i++) kentta_jono += kentat[i] + ",";
      kentta_jono += kentat[kentat.length-1];
      lauseke = yhteys.prepareStatement("INSERT INTO "+ taulu +" ("+ kentta_jono +") VALUES (?,?,?,?,?,?,?,?)");
      
      for (int i=0;i<kentat.length;i++) {
        parametri = req.getParameter(kentat[i]);
        if (parametri == null) parametri = "Ei";
        lauseke.setString(i+1, parametri);
      }

      lauseke.executeUpdate();
      
      out.println("<H2>Tietosi tallennettiin onnistuneesti!</H2>");
    }
    catch (SQLException e) { out.println("SQL-poikkeus: " +e.getMessage()); }
    finally {
      try { if (yhteys != null) yhteys.close(); }
      catch (SQLException e2) { out.println("Sulkeminen eponnistui"); }	
    }
  }
  
  private void haeVieraskirja(PrintWriter out)
  {
    Connection yhteys = null;
    Statement lauseke = null;
    ResultSet tulos = null;

    try {
      Class.forName(ajuri).newInstance();
    } catch (Exception ee) { out.println("Virhe: " +ee.getMessage()); }  
    try {
      yhteys = DriverManager.getConnection(tietokanta);
      lauseke = yhteys.createStatement();
      
      tulos = lauseke.executeQuery("SELECT Nimi, Email, Linkki, Viesti FROM "+ taulu +" ORDER BY Nimi");
      out.println("<H2>Vieraskirja:</H2>");
      while (tulos.next())
      {
      	String laitetaankoLinkki = tulos.getString("Linkki").trim();
        if (laitetaankoLinkki.equals("Kyll")) 
          out.println("<a href=mailto:" + tulos.getString("Email") + ">" + tulos.getString("Nimi") + "</a>");
        else 
          out.println(tulos.getString("Nimi") + "</a>");
        out.println(" - " + tulos.getString("Viesti") + "<BR>");

      }
    }
    catch (SQLException e) { out.println("SQLPoikkeus: " +e.getMessage()); }
    finally {
      try { if (yhteys != null) yhteys.close(); }
      catch (SQLException e2) { out.println("Sulkeminen eponnistui"); }	
    }
  }

}